Info<< "Propeller Thrust calculation" << endl;

// Allocate the body force vector field and initialize it to zero
volVectorField PropThrust
(
    IOobject
    (
        "PropThrust",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    mesh,
    dimensionedVector("PropThrust", dimensionSet(1,-2,-2,0,0,0,0), Foam::vector(0,0,0))
);

// Read a dictionary to configure the body force
IOdictionary bodyForceDict
(
    IOobject
    (
        "bodyForceDict",
        runTime.time().constant(),
        runTime,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    )
);

IOdictionary dragForceDict
(
    IOobject
    (
        "dragForceDict",
        runTime.time().constant(),
        runTime,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    )
);

// Define the (cylindrical) region for the body force from dictionary values
const scalar yProp (readScalar(bodyForceDict.lookup("yProp")));
const scalar zProp (readScalar(bodyForceDict.lookup("zProp")));
const scalar xPropmin (readScalar(bodyForceDict.lookup("xPropmin")));
const scalar xPropmax (readScalar(bodyForceDict.lookup("xPropmax")));
const scalar tipRadius (readScalar(bodyForceDict.lookup("tipRadius")));
const scalar hubRadius (readScalar(bodyForceDict.lookup("hubRadius")));

forces F
 (
   "forces",
    mesh,
    dragForceDict
 );
F.calcForcesMoment();
vector force =F.forceEff();
scalar drag = force.x();
Info << "Thrust = " << drag << endl;
scalar thrust = 1.1*drag; //Considering 10% more

//Total Volume of the propeller disc using cell integration
label numCells(0),numInside(0);
scalar integratedVolume(0.0);
//scalar analyticalTotalVolume(3.1415*(tipRadius*tipRadius-hubRadius*hubRadius)*(xPropmax-xPropmin)/2);

forAll(mesh.cells(),cellID)
{
    // Loop over all the cells in the mesh
++numCells;
const vector& cellCenter = mesh.C()[cellID];
const scalar dy = cellCenter[1] - yProp;
const scalar dz = cellCenter[2] - zProp;
const vector r(0,dy,dz);
if ((pos(xPropmax-cellCenter[0])) &&
   (pos(cellCenter[0]-xPropmin)) &&
   (pos(tipRadius-mag(r))) &&
   (pos(mag(r)-hubRadius)))
   {
     ++numInside;
     integratedVolume += mesh.V()[cellID];
   }
}
       
// Reduce values for parallel execution
reduce(numInside,sumOp<label>());
reduce(numCells,sumOp<label>());
reduce(integratedVolume,sumOp<scalar>());
Info << "integratedVolume              ="<< integratedVolume<< endl;
//Info << "AnalyticalVolume              =" << analyticalTotalVolume << endl;

// Radial Distribution of the thrust
scalar dt = 0.0;
scalar rstar = 0.0;
scalar integratedThrust = 0.0;
scalar tdist = 0.0;


forAll(mesh.cells(),cellID)
{
 ++numCells;
 const vector& cellCenter = mesh.C()[cellID];
 const scalar dy = cellCenter[1] - yProp;
 const scalar dz = cellCenter[2] - zProp;
 const vector r(0,dy,dz);
 if ((pos(xPropmax-cellCenter[0])) &&
 (pos(cellCenter[0]-xPropmin)) &&
 (pos(tipRadius-mag(r))) &&
 (pos(mag(r)-hubRadius)))
{
 rstar=(mag(r)-hubRadius)/(tipRadius-hubRadius);
 tdist = 1.0;
 tdist = 3.5183*rstar*::sqrt(1.0-rstar);
 dt = thrust/(integratedVolume)*tdist;
 vector tempForce(dt, 0, 0);
 PropThrust[cellID] =tempForce; 
 integratedThrust += dt * mesh.V()[cellID]; 
}
}
reduce(integratedThrust,sumOp<scalar>());

Info << "integratedThrust              = " << integratedThrust << endl;
